	/*  
	
	procs.sql -	PL/SQL Procedures used in Collaboration Area Data Integrity Check 

	Copyright 2005, IBM Corporation
	Version August 17, 2005

	*/
CREATE OR REPLACE PROCEDURE COUNT_SHOW_SOME_PK_VIEW(View_Name VARCHAR2, Description VARCHAR2)
IS
	TYPE MyCurTyp IS REF CURSOR;
	my_cv MyCurTyp;
	TYPE view_rec_type IS RECORD(itm_primary_key VARCHAR2(300)); -- dependent on ITM ddl
	view_rec view_rec_type;
	sql_stmt VARCHAR2(200);
	rows_fetched NUMBER;
   	TYPE primary_key_tab_type is table of VARCHAR2(300) index by binary_integer;
	primary_key_tab primary_key_tab_type;
	KEYS_TO_SHOW NUMBER := 5;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** REPORT FOR VIEW ' || View_Name || ', Description = ' || Description);
	sql_stmt := 'SELECT * FROM ' || LTRIM(RTRIM(View_Name));

	-- Report total count	
	OPEN my_cv FOR sql_stmt;
	FETCH my_cv BULK COLLECT INTO primary_key_tab;
	rows_fetched := my_cv%ROWCOUNT;
	DBMS_OUTPUT.PUT_LINE('Total number of primary keys in view = ' || rows_fetched);
	CLOSE my_cv;

	-- show some primary keys
	OPEN my_cv FOR sql_stmt;
	LOOP
		FETCH my_cv INTO view_rec;
		EXIT WHEN my_cv%NOTFOUND;
		EXIT WHEN my_cv%ROWCOUNT > KEYS_TO_SHOW;
		DBMS_OUTPUT.PUT_LINE('Next primary key in this view = ' || view_rec.itm_primary_key);
	END LOOP;
	CLOSE my_cv;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc COUNT_SHOW_SOME_PK_VIEW('||View_Name||', '||Description||')');
END;
/
CREATE OR REPLACE PROCEDURE COUNT_SHOW_SOME_ITEM_ID_VIEW(View_Name VARCHAR2, Description VARCHAR2)
IS
	TYPE MyCurTyp IS REF CURSOR;
	my_cv MyCurTyp;
	TYPE view_rec_type IS RECORD(itm_id NUMBER(9)); -- dependent on ITM ddl
	view_rec view_rec_type;
	sql_stmt VARCHAR2(200);
	rows_fetched NUMBER;
   	TYPE item_id_tab_type is table of NUMBER(9) index by binary_integer;
	item_id_tab item_id_tab_type;
	KEYS_TO_SHOW NUMBER := 5;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** REPORT FOR VIEW ' || View_Name || ', Description = ' || Description);
	sql_stmt := 'SELECT * FROM ' || LTRIM(RTRIM(View_Name));

	-- Report total count	
	OPEN my_cv FOR sql_stmt;
	FETCH my_cv BULK COLLECT INTO item_id_tab;
	rows_fetched := my_cv%ROWCOUNT;
	DBMS_OUTPUT.PUT_LINE('Total number of item ids in view = ' || rows_fetched);
	CLOSE my_cv;

	-- show some item ids
	OPEN my_cv FOR sql_stmt;
	LOOP
		FETCH my_cv INTO view_rec;
		EXIT WHEN my_cv%NOTFOUND;
		EXIT WHEN my_cv%ROWCOUNT > KEYS_TO_SHOW;
		DBMS_OUTPUT.PUT_LINE('Next item id in this view = ' || view_rec.itm_id);
	END LOOP;
	CLOSE my_cv;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc COUNT_SHOW_SOME_ITEM_ID_VIEW('||View_Name||', '||Description||')');
END;
/

CREATE OR REPLACE PROCEDURE SHOW_ALL_PK_VIEW(View_Name VARCHAR2, Description VARCHAR2)
IS
	TYPE MyCurTyp IS REF CURSOR;
	my_cv MyCurTyp;
	TYPE view_rec_type IS RECORD(itm_primary_key VARCHAR2(300)); -- dependent on ITM ddl
	view_rec view_rec_type;
	sql_stmt VARCHAR2(200);
BEGIN
	sql_stmt := 'SELECT * FROM ' || LTRIM(RTRIM(View_Name));

	-- show all primary keys
	OPEN my_cv FOR sql_stmt;
	LOOP
		FETCH my_cv INTO view_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE(view_rec.itm_primary_key);
	END LOOP;
	CLOSE my_cv;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc SHOW_ALL_PRIMARY_KEY_VIEW('||View_Name||', '||Description||')');
END;
/


CREATE OR REPLACE PROCEDURE SHOW_ALL_IID_VIEW(View_Name VARCHAR2, Description VARCHAR2)
IS
	TYPE MyCurTyp IS REF CURSOR;
	my_cv MyCurTyp;
	TYPE view_rec_type IS RECORD(itm_id NUMBER(9)); -- dependent on ITM ddl
	view_rec view_rec_type;
	sql_stmt VARCHAR2(200);
BEGIN
	sql_stmt := 'SELECT * FROM ' || LTRIM(RTRIM(View_Name));

	-- show all item ids
	OPEN my_cv FOR sql_stmt;
	LOOP
		FETCH my_cv INTO view_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE(view_rec.itm_id);
	END LOOP;
	CLOSE my_cv;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc SHOW_ALL_IID_VIEW('||View_Name||', '||Description||')');
END;
/


CREATE OR REPLACE FUNCTION GET_ITEM_ID(catalog_id INTEGER, pk VARCHAR2) RETURN NUMBER IS
    out_id NUMBER := 0;
    TYPE cur_typ IS REF CURSOR;
    c cur_typ;
    query_str VARCHAR2(200);
BEGIN
    query_str := 'select distinct itm_id from itm where itm_primary_key = :pk and itm_container_id = :cid';
    OPEN c FOR query_str USING pk, catalog_id;
    LOOP
        FETCH c INTO out_id;
        EXIT WHEN c%NOTFOUND;
    END LOOP;
    CLOSE c;
   RETURN out_id;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END GET_ITEM_ID;
/


CREATE OR REPLACE FUNCTION GET_ITEM_PRIMARY_KEY(catalog_id INTEGER, item_id NUMBER) RETURN VARCHAR2 IS
    out_pk VARCHAR2(300) := ' '; -- default empty pk
    TYPE cur_typ IS REF CURSOR;
    c cur_typ;
    query_str VARCHAR2(200);
BEGIN
    query_str := 'select distinct itm_primary_key from itm where itm_id = :iid and itm_container_id = :cid';
    OPEN c FOR query_str USING item_id, catalog_id;
    LOOP
        FETCH c INTO out_pk;
        EXIT WHEN c%NOTFOUND;
    END LOOP;
    CLOSE c;
   RETURN out_pk;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END GET_ITEM_PRIMARY_KEY;
/


CREATE OR REPLACE PROCEDURE BACKUP_ITEM_NEW(clnCoaId INTEGER, primary_key VARCHAR2, src_item_id NUMBER, ca_item_id NUMBER)
IS
-- Backup an item by pk, both from collaboration area and source catalog
-- Item may be in only one or both.  Specified by positive-valued ids passed.
BEGIN
  -- DBMS_OUTPUT.PUT_LINE(' ');
  DBMS_OUTPUT.PUT_LINE('   STARTING BACKUP_ITEM('||primary_key||', '||src_item_id||', '||ca_item_id||') ...');

  -- Do source item first 
  IF (src_item_id > 0) THEN
    DBMS_OUTPUT.PUT_LINE('     BACKUP SOURCE ITEM ID='||src_item_id);
    insert into ITMRSV_CLNCOA (select * from itm where itm_id = src_item_id );
    insert into CEM_CLNCOA  (select * from cem where cem_src_entry_id = src_item_id );
    insert into CEH_CLNCOA (select * from ceh where ceh_entry_key = primary_key);
  END IF;

  -- DO CA item
  IF (ca_item_id > 0) THEN
    DBMS_OUTPUT.PUT_LINE('     BACKUP CA ITEM ID='||ca_item_id);
    execute immediate 'insert into ITM_CLNCOA (select * from itm where itm_id = ' || ca_item_id || ')' ;
    execute immediate 'insert into ITA_CLNCOA (select * from ita where ita_item_id = ' || ca_item_id || ')' ;
    execute immediate 'insert into ICM_CLNCOA (select * from icm where icm_item_id = ' || ca_item_id || ')' ;
    execute immediate 'insert into CEM_CLNCOA (select * from cem where cem_ca_entry_id = ' || ca_item_id || ')' ;
    execute immediate 'insert into CAE_CLNCOA (select * from cae where cae_entry_id = ' || ca_item_id || ')' ;
    insert into CEH_CLNCOA (select * from ceh where ceh_entry_key = primary_key);
    execute immediate 'insert into CEL_CLNCOA (select * from cel where cel_entry_id = ' || ca_item_id || ')' ;
    insert into WEE_CLNCOA (select clnCoaId, wee_event_id, wee_entry_id, wee_event_entry_status from wee where wee_entry_id = ca_item_id);
  END IF;
  DBMS_OUTPUT.PUT_LINE('   FINISHED BACKUP_ITEM('||primary_key||', '||src_item_id||', '||ca_item_id||')');
-- 
EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc BACKUP_ITEM_NEW('||primary_key||', '||src_item_id||', '||ca_item_id||')');
END;
/

CREATE OR REPLACE PROCEDURE BACKUP_CA_ITEM(clnCoaId INTEGER, primary_key VARCHAR2, ca_item_id NUMBER)
IS
-- Backup an item known in a collaboration area (i.e. not a source catalog item)
-- 
-- primary_key may be NULL, but ca_item_id should not be.
--
BEGIN
	--DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('   STARTING BACKUP_CA_ITEM('||primary_key||', '||ca_item_id||') ...');

	-- DO CA item
	IF (ca_item_id > 0) THEN
		DBMS_OUTPUT.PUT_LINE('     BACKUP CA ITEM ID='||ca_item_id);
    insert into ITM_CLNCOA (select * from itm where itm_id = ca_item_id);
    insert into ITA_CLNCOA (select * from ita where ita_item_id = ca_item_id);
    insert into ICM_CLNCOA (select * from icm where icm_item_id = ca_item_id);
    insert into CEM_CLNCOA (select * from cem where cem_ca_entry_id = ca_item_id);
    insert into CAE_CLNCOA (select * from cae where cae_entry_id = ca_item_id);
    insert into CEH_CLNCOA (select * from ceh where ceh_entry_key = primary_key);
    insert into CEL_CLNCOA (select * from cel where cel_entry_id = ca_item_id);
    insert into WEE_CLNCOA (select clnCoaId, wee_event_id, wee_entry_id, wee_event_entry_status from wee where wee_entry_id = ca_item_id);
	END IF;

	DBMS_OUTPUT.PUT_LINE('   FINISHED BACKUP_CA_ITEM('||primary_key||', '||ca_item_id||')');

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc BACKUP_CA_ITEM('||primary_key||', '||ca_item_id||')');
END;
/


CREATE OR REPLACE PROCEDURE BACKUP_SRC_ITEM(clnCoaId INTEGER, primary_key VARCHAR2, src_item_id NUMBER)
IS
-- Backup an item known to be in a source catalog (i.e. not a collaboration area item).
-- 
-- primary_key may be NULL, but src_item_id should not be.
--
BEGIN
	-- DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('   STARTING BACKUP_SRC_ITEM('||primary_key||', '||src_item_id||') ...');

	-- Do source item first 
	IF (src_item_id > 0) THEN
		DBMS_OUTPUT.PUT_LINE('     BACKUP SOURCE ITEM ID='||src_item_id);
    execute immediate 'insert into ITMRSV_CLNCOA (select * from itm where itm_id = ' || src_item_id || ' and itm_next_version_id=999999999)' ;
    execute immediate 'insert into CEM_CLNCOA  (select * from cem where cem_src_entry_id = ' || src_item_id || ')' ;
    insert into CEH_CLNCOA (select * from ceh where ceh_entry_key = primary_key);
	END IF;

	DBMS_OUTPUT.PUT_LINE('   FINISHED BACKUP_SRC_ITEM('||primary_key||', '||src_item_id||')');

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc BACKUP_SRC_ITEM('||primary_key||', '||src_item_id||')');
END;
/


CREATE OR REPLACE PROCEDURE CLEAN_ITEM(clnCoaId INTEGER, primary_key VARCHAR2, src_item_id NUMBER, ca_item_id NUMBER)
IS
-- CLEAN an item by pk, both from collaboration area and source catalog
-- Item may be in only one or both.  Specified by positive-valued ids passed.
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('STARTING CLEAN_ITEM('||primary_key||', '||src_item_id||', '||ca_item_id||') ...');

        BACKUP_ITEM_NEW(clnCoaId, primary_key, src_item_id, ca_item_id);

	-- Do source item first 
	IF (src_item_id > 0) THEN
		DBMS_OUTPUT.PUT_LINE('     CLEANING SOURCE ITEM ID='||src_item_id);
		update itm set itm_collab_info = ''  where itm_id = src_item_id and itm_next_version_id=999999999;
		delete from cem where cem_src_entry_id = src_item_id;
	END IF;

	-- DO CA item
	IF (ca_item_id > 0) THEN
		DBMS_OUTPUT.PUT_LINE('     CLEANING CA ITEM ID='||ca_item_id);
		delete from itm where itm_id = ca_item_id;
		delete from ita where ita_item_id = ca_item_id;
		delete from icm where icm_item_id = ca_item_id;
		delete from cem where cem_ca_entry_id = ca_item_id;
		delete from cae where cae_entry_id = ca_item_id;
		delete from ceh where ceh_entry_key = primary_key;
		delete from cel where cel_entry_id = ca_item_id;
		delete from wee where wee_entry_id = ca_item_id;
	END IF;

	DBMS_OUTPUT.PUT_LINE('FINISHED CLEAN_ITEM('||primary_key||', '||src_item_id||', '||ca_item_id||')');

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc CLEAN_ITEM('||primary_key||', '||src_item_id||', '||ca_item_id||')');
END;
/
CREATE OR REPLACE PROCEDURE CLEAN_CA_ITEM(clnCoaId INTEGER, primary_key VARCHAR2, ca_item_id NUMBER)
IS
-- CLEAN an item known in a collaboration area (i.e. not a source catalog item)
--
-- WARNING: be careful not to pass in the id of an item in a source catalog; it will be deleted !!!!!!!
--
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('STARTING CLEAN_CA_ITEM('||primary_key||', '||ca_item_id||') ...');

        BACKUP_CA_ITEM(clnCoaId, primary_key, ca_item_id);

	-- DO CA item
	IF (ca_item_id > 0) THEN
		DBMS_OUTPUT.PUT_LINE('     CLEANING CA ITEM ID='||ca_item_id);
		delete from itm where itm_id = ca_item_id;
		delete from ita where ita_item_id = ca_item_id;
		delete from icm where icm_item_id = ca_item_id;
		delete from cem where cem_ca_entry_id = ca_item_id;
		delete from cae where cae_entry_id = ca_item_id;
		delete from ceh where ceh_entry_key = primary_key;
		delete from cel where cel_entry_id = ca_item_id;
		delete from wee where wee_entry_id = ca_item_id;
	END IF;

	DBMS_OUTPUT.PUT_LINE('FINISHED CLEAN_CA_ITEM('||primary_key||', '||ca_item_id||')');

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc CLEAN_CA_ITEM('||primary_key||', '||ca_item_id||')');
END;
/
CREATE OR REPLACE PROCEDURE CLEAN_SRC_ITEM(clnCoaId INTEGER, primary_key VARCHAR2, src_item_id NUMBER)
IS
-- CLEAN an item known to be in a source catalog (i.e. not a collaboration area item).
--
-- Passing a CA item by mistake should be benign.
--
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('STARTING CLEAN_SRC_ITEM('||primary_key||', '||src_item_id||') ...');

        BACKUP_SRC_ITEM(clnCoaId, primary_key, src_item_id);

	-- Do source item first 
	IF (src_item_id > 0) THEN
		DBMS_OUTPUT.PUT_LINE('     CLEANING SOURCE ITEM ID='||src_item_id);
		update itm set itm_collab_info = ''  where itm_id = src_item_id and itm_next_version_id=999999999;
		delete from cem where cem_src_entry_id = src_item_id;
	END IF;

	DBMS_OUTPUT.PUT_LINE('FINISHED CLEAN_SRC_ITEM('||primary_key||', '||src_item_id||')');

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc CLEAN_SRC_ITEM('||primary_key||', '||src_item_id||')');
END;
/
CREATE OR REPLACE PROCEDURE CLEAN_ITEMS_OF_PK_VIEW(View_Name VARCHAR2, Collab_Id INTEGER, src_catalog_id INTEGER, ca_catalog_id INTEGER)
IS
	TYPE MyCurTyp IS REF CURSOR;
	my_cv MyCurTyp;
	TYPE view_rec_type IS RECORD(itm_primary_key VARCHAR2(300)); -- dependent on ITM ddl
	view_rec view_rec_type;
	sql_stmt VARCHAR2(200);
	src_item_id number(9);
	ca_item_id number(9);
	pk VARCHAR2(300);
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*************************************************');
	DBMS_OUTPUT.PUT_LINE('STARTING CLEAN_ITEMS_OF_PK_VIEW('||View_Name||', '||Collab_Id||', '
		||src_catalog_id||', '||ca_catalog_id||') ...');
	DBMS_OUTPUT.PUT_LINE(' ');
	sql_stmt := 'SELECT * FROM ' || LTRIM(RTRIM(View_Name));

	-- do all primary keys
	OPEN my_cv FOR sql_stmt;
	LOOP
		FETCH my_cv INTO view_rec;
		EXIT WHEN my_cv%NOTFOUND;
		pk := view_rec.itm_primary_key;
		src_item_id := GET_ITEM_ID(src_catalog_id, pk);
		ca_item_id := GET_ITEM_ID(ca_catalog_id, pk);
		CLEAN_ITEM(Collab_Id, pk, src_item_id, ca_item_id);
	END LOOP;
	CLOSE my_cv;
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*************************************************');
	DBMS_OUTPUT.PUT_LINE('FINISHED CLEAN_ITEMS_OF_PK_VIEW('||View_Name||', '||Collab_Id||', '
		||src_catalog_id||', '||ca_catalog_id||')');
	DBMS_OUTPUT.PUT_LINE(' ');
EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc CLEAN_ITEMS_OF_PK_VIEW('||View_Name||', '||Collab_Id||', '
		||src_catalog_id||', '||ca_catalog_id||')');
END;
/
CREATE OR REPLACE PROCEDURE CLEAN_CA_ITEMS_OF_IID_VIEW(View_Name VARCHAR2, Collab_Id INTEGER, ca_catalog_id INTEGER)
IS
--
-- WARNING: BE SURE View_Name is an ITEM_ID_VIEW consisting of CA, not source, Items !!!!!
--		Else you risk deleting source catalog items !!!!!
--
	TYPE MyCurTyp IS REF CURSOR;
	my_cv MyCurTyp;
	TYPE view_rec_type IS RECORD(itm_id NUMBER(9)); -- dependent on ITM ddl
	view_rec view_rec_type;
	sql_stmt VARCHAR2(200);
	ca_item_id NUMBER(9);
	pk VARCHAR2(300);
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*************************************************');
	DBMS_OUTPUT.PUT_LINE('STARTING CLEAN_CA_ITEMS_OF_IID_VIEW('||View_Name||', '||Collab_Id||', '
		||ca_catalog_id||') ...');
	DBMS_OUTPUT.PUT_LINE(' ');
	sql_stmt := 'SELECT * FROM ' || LTRIM(RTRIM(View_Name));

	-- do all item ids
	OPEN my_cv FOR sql_stmt;
	LOOP
		FETCH my_cv INTO view_rec;
		EXIT WHEN my_cv%NOTFOUND;
		ca_item_id := view_rec.itm_id;
		pk := GET_ITEM_PRIMARY_KEY(ca_catalog_id, ca_item_id);
		CLEAN_CA_ITEM(Collab_Id, pk, ca_item_id);
	END LOOP;
	CLOSE my_cv;
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*************************************************');
	DBMS_OUTPUT.PUT_LINE('FINISHED CLEAN_CA_ITEMS_OF_IID_VIEW('||View_Name||', '||Collab_Id||', '
		||ca_catalog_id||')');
	DBMS_OUTPUT.PUT_LINE(' ');
EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc CLEAN_CA_ITEMS_OF_IID_VIEW('||View_Name||', '||Collab_Id||', '
		||ca_catalog_id||')');
END;
/
CREATE OR REPLACE PROCEDURE CLEAN_SRC_ITEMS_OF_IID_VIEW(View_Name VARCHAR2, Collab_Id INTEGER, src_catalog_id INTEGER)
IS
--
-- View_Name is an ITEM_ID_VIEW consisting of source, not CA, Items.
--
	TYPE MyCurTyp IS REF CURSOR;
	my_cv MyCurTyp;
	TYPE view_rec_type IS RECORD(itm_id NUMBER(9)); -- dependent on ITM ddl
	view_rec view_rec_type;
	sql_stmt VARCHAR2(200);
	src_item_id NUMBER(9);
	pk VARCHAR2(300);
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*************************************************');
	DBMS_OUTPUT.PUT_LINE('STARTING CLEAN_SRC_ITEMS_OF_IID_VIEW('||View_Name||', '||Collab_Id||', '
		||src_catalog_id||') ...');
	DBMS_OUTPUT.PUT_LINE(' ');
	sql_stmt := 'SELECT * FROM ' || LTRIM(RTRIM(View_Name));

	-- do all item ids
	OPEN my_cv FOR sql_stmt;
	LOOP
		FETCH my_cv INTO view_rec;
		EXIT WHEN my_cv%NOTFOUND;
		src_item_id := view_rec.itm_id;
		pk := GET_ITEM_PRIMARY_KEY(src_catalog_id, src_item_id);
		CLEAN_SRC_ITEM(Collab_Id, pk, src_item_id);
	END LOOP;
	CLOSE my_cv;
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*************************************************');
	DBMS_OUTPUT.PUT_LINE('FINISHED CLEAN_SRC_ITEMS_OF_IID_VIEW('||View_Name||', '||Collab_Id||', '
		||src_catalog_id||')');
	DBMS_OUTPUT.PUT_LINE(' ');
EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc CLEAN_SRC_ITEMS_OF_IID_VIEW('||View_Name||', '||Collab_Id||', '
		||src_catalog_id||')');
END;
/


CREATE OR REPLACE PROCEDURE RESTORE_itmrsv(clnSrcCtgId INTEGER)
IS
 -- assumes source items only checked out to one collab area, because restores all items of a given source catalog.
	CURSOR my_cv is select * from itmrsv_clncoa where itm_container_id = clnSrcCtgId;
	itm_rec itmrsv_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF itmrsv TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO itm_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE('Next itm row = ' || itm_rec.itm_id||', '||itm_rec.itm_collab_info);
	
		-- select existing row if any
		delete from itm where (                                        
    itm_company_id = itm_rec.itm_company_id and
    itm_id = itm_rec.itm_id  and
    itm_container_id = itm_rec.itm_container_id  and
    itm_primary_key  = itm_rec.itm_primary_key  and
    itm_version_id = itm_rec.itm_version_id and
    itm_next_version_id = itm_rec.itm_next_version_id 
    -- and itm_status = itm_rec.status 
    -- and itm_collab_info  = itm_rec.itm_collab_info --- XXX varchar2(3000) compare?
		); 
		
		-- restore row
		insert into itm values (                                       
    itm_rec.itm_company_id,
    itm_rec.itm_id  ,
    itm_rec.itm_container_id  ,
    itm_rec.itm_primary_key  ,
    itm_rec.itm_version_id ,
    itm_rec.itm_next_version_id  ,
    itm_rec.itm_status  ,
    itm_rec.itm_collab_info
		); 

		
	END LOOP;


	CLOSE my_cv;
	commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_itm');
END;
/


CREATE OR REPLACE PROCEDURE RESTORE_itm(clnCoaCtgId INTEGER)
IS
	CURSOR my_cv is select * from itm_clncoa where itm_container_id = clnCoaCtgId;
	itm_rec itm_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF itm TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO itm_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE('Next itm row = ' || itm_rec.itm_container_id||', '||itm_rec.itm_primary_key);
	
		-- delete existing row if any
		delete from itm where (                                        
    itm_company_id = itm_rec.itm_company_id and
    itm_id = itm_rec.itm_id  and
    itm_container_id = itm_rec.itm_container_id  and
    itm_primary_key  = itm_rec.itm_primary_key  and
    itm_version_id = itm_rec.itm_version_id and
    itm_next_version_id = itm_rec.itm_next_version_id 
    -- and itm_status = itm_rec.itm_status
    -- and itm_collab_info  = itm_rec.itm_collab_info --- XXX varchar2(3000) compare?
		); 
		
		-- restore row
		insert into itm values (                                       
    itm_rec.itm_company_id,
    itm_rec.itm_id  ,
    itm_rec.itm_container_id  ,
    itm_rec.itm_primary_key  ,
    itm_rec.itm_version_id ,
    itm_rec.itm_next_version_id  ,
    itm_rec.itm_status  ,
    itm_rec.itm_collab_info
		); 

		
	END LOOP;


	CLOSE my_cv;
	commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_itm');
END;
/




CREATE OR REPLACE PROCEDURE RESTORE_ita(clnCoaCtgId INTEGER)
IS
	CURSOR my_cv is select * from ita_clncoa where ita_catalog_id = clnCoaCtgId;
	ita_rec ita_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF ita TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO ita_rec;
		EXIT WHEN my_cv%NOTFOUND;
        -- no confirm output line; generates excess amount of output for Gap; 10/2/05 Dennis
		-- DBMS_OUTPUT.PUT_LINE('Next ita row = ' || ita_rec.ita_item_id||', '||ita_rec.ita_node_id);
	
		-- delete existing row if any
		delete from ita where (                                        
    ita_company_id = ita_rec.ita_company_id and
    ita_catalog_id = ita_rec.ita_catalog_id  and
    ita_item_id = ita_rec.ita_item_id  and
    ita_node_id  = ita_rec.ita_node_id  and
    -- ita_value_numeric = ita_rec.ita_value_numeric and
    ita_value_string = ita_rec.ita_value_string  and  
    -- ita_value_string_search = ita_rec.ita_value_string_search  and
    -- ita_value_date  = ita_rec.ita_value_date and  -- XXX date compare
    ita_version_id = ita_rec.ita_version_id  and
    ita_next_version_id  = ita_rec.ita_next_version_id  and
    ita_order = ita_rec.ita_order and
    ita_parent_order = ita_rec.ita_parent_order and  
    ita_row_order = ita_rec.ita_row_order and
    ita_mod_user_id  = ita_rec.ita_mod_user_id
    -- and ita_mod_date = ita_rec.ita_mod_date  -- XXX why date compare not work?
		); 
		
		-- restore row
		insert into ita values (                                       
    ita_rec.ita_company_id,
    ita_rec.ita_catalog_id  ,
    ita_rec.ita_item_id  ,
    ita_rec.ita_node_id  ,
    ita_rec.ita_value_numeric ,
    ita_rec.ita_value_string  ,
    ita_rec.ita_value_string_search  ,
    ita_rec.ita_value_date  ,
    ita_rec.ita_version_id ,
    ita_rec.ita_next_version_id,
    ita_rec.ita_order,
    ita_rec.ita_parent_order,  
    ita_rec.ita_row_order,
    ita_rec.ita_mod_user_id,
    ita_rec.ita_mod_date 
		); 

		
	END LOOP;


	CLOSE my_cv;
	commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_ita');
END;
/




CREATE OR REPLACE PROCEDURE RESTORE_icm(clnCoaCtgId INTEGER)
IS
	CURSOR my_cv is select * from icm_clncoa where icm_catalog_id = clnCoaCtgId;
	icm_rec icm_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF icm TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO icm_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE('Next icm row = ' || icm_rec.icm_item_id);
	
		-- delete existing rowand if any
		delete from icm where (                                        
    icm_company_id = icm_rec.icm_company_id and
    icm_catalog_id = icm_rec.icm_catalog_id  and
    icm_item_id = icm_rec.icm_item_id  and
    icm_cat_tree_id  = icm_rec.icm_cat_tree_id  and
	icm_category_id  = icm_rec.icm_category_id  and
	icm_version_id  = icm_rec.icm_version_id  and
    icm_next_version_id = icm_rec.icm_next_version_id 
		); 
		
		-- restore row
		insert into icm values (                                       
    icm_rec.icm_company_id,
    icm_rec.icm_catalog_id  ,
    icm_rec.icm_item_id  ,
    icm_rec.icm_cat_tree_id  ,
icm_rec.icm_category_id,
icm_rec.icm_version_id,
    icm_rec.icm_next_version_id 
		); 
	END LOOP;
	CLOSE my_cv;
    commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_icm');
END;
/


CREATE OR REPLACE PROCEDURE RESTORE_cel(clnCoaId INTEGER)
IS
	CURSOR my_cv is select * from cel_clncoa where cel_container_id = clnCoaId;
		-- "container" id here is actually the CA id
	cel_rec cel_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF cel TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO cel_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE('Next cel row = ' || cel_rec.cel_entry_id);
	
		-- delete existing row if any
		delete from cel where (                                        
    cel_cmp_id = cel_rec.cel_cmp_id and
    cel_id = cel_rec.cel_id  and
    cel_user_id = cel_rec.cel_user_id  and                  
    -- cel_time_acquired = cel_rec.cel_time_acquired  and      -- XXX Date compare not working?  
    -- cel_time_of_expiry = cel_rec.cel_time_of_expiry   and    -- XXX Date compare not working?       
    cel_container_id = cel_rec.cel_container_id  and         
    cel_container_type = cel_rec.cel_container_type  and 
    cel_entry_id = cel_rec.cel_entry_id  and
    cel_step_path_id = cel_rec.cel_step_path_id 
		); 
		
		-- restore row
		insert into cel values (                                       
    	cel_rec.cel_cmp_id,
    	cel_rec.cel_id  ,
	cel_rec.cel_user_id,
	cel_rec.cel_time_acquired,
	cel_rec.cel_time_of_expiry,
	cel_rec.cel_container_id,
	cel_rec.cel_container_type,
    	cel_rec.cel_entry_id  ,
    	cel_rec.cel_step_path_id 
		); 
	END LOOP;
	CLOSE my_cv;
    commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_cel');
END;
/


CREATE OR REPLACE PROCEDURE RESTORE_cae(clnCoaId INTEGER)
IS
	CURSOR my_cv is select * from cae_clncoa where cae_ca_id = clnCoaId;
	cae_rec cae_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF cae TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO cae_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE('Next cae row = ' || cae_rec.cae_step_path_id);
	
		-- delete existing row if any
		delete from cae where (                                        
    cae_cmp_id = cae_rec.cae_cmp_id and
    cae_ca_id = cae_rec.cae_ca_id  and
    cae_entry_id = cae_rec.cae_entry_id  and
    cae_step_path_id  = cae_rec.cae_step_path_id  and
    cae_deadline_step_path_id = cae_rec.cae_deadline_step_path_id and
    --cae_deadline = cae_rec.cae_deadline  and  -- dont use date XXX why 
    cae_data = cae_rec.cae_data  and
    cae_available  = cae_rec.cae_available  and
    cae_lock_id = cae_rec.cae_lock_id
		); 
		
		-- restore row
		insert into cae values (                                       
    cae_rec.cae_cmp_id,
    cae_rec.cae_ca_id  ,
    cae_rec.cae_entry_id  ,
    cae_rec.cae_step_path_id  ,
    cae_rec.cae_deadline_step_path_id ,
    cae_rec.cae_deadline  ,
    cae_rec.cae_data  ,
    cae_rec.cae_available  ,
    cae_rec.cae_lock_id 
		); 

		
	END LOOP;


	CLOSE my_cv;
	commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_cae');
END;
/



CREATE OR REPLACE PROCEDURE RESTORE_cem(clnCoaId INTEGER)
IS
	CURSOR my_cv is select * from cem_clncoa where cem_ca_id = clnCoaId;
	cem_rec cem_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF cem TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO cem_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE('Next cem row = ' || cem_rec.cem_ca_entry_id);
	
		-- delete existing row if any
		delete from cem where (                                        
    cem_cmp_id = cem_rec.cem_cmp_id and
    cem_ca_id = cem_rec.cem_ca_id  and
    cem_ca_entry_id = cem_rec.cem_ca_entry_id  and
    cem_src_ctg_id  = cem_rec.cem_src_ctg_id  and
    cem_src_entry_id = cem_rec.cem_src_entry_id 
		); 
		
		-- restore row
		insert into cem values (                                       
    cem_rec.cem_cmp_id,
    cem_rec.cem_ca_id  ,
    cem_rec.cem_ca_entry_id  ,
    cem_rec.cem_src_ctg_id  ,
    cem_rec.cem_src_entry_id 
		); 
	END LOOP;
	CLOSE my_cv;
    commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_cem');
END;
/



CREATE OR REPLACE PROCEDURE RESTORE_ceh(clnCAName VARCHAR)
IS
	CURSOR my_cv is select * from ceh_clncoa where ceh_ca_name = clnCAName;
	ceh_rec ceh_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF ceh TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO ceh_rec;
		EXIT WHEN my_cv%NOTFOUND;
        -- no confirm output line; generates excess amount of output for Gap; 10/2/05 Dennis
		-- DBMS_OUTPUT.PUT_LINE('Next ceh row = ' || ceh_rec.ceh_event||', '||ceh_rec.ceh_entry_key);
	
		-- delete existing row if any
		delete from ceh where (                                        
    ceh_cmp_id = ceh_rec.ceh_cmp_id and
    ceh_id = ceh_rec.ceh_id   -- dont need anything more
    --and ceh_date = ceh_rec.ceh_date  and   -- XXX date compare
    -- ceh_entry_key  = ceh_rec.ceh_entry_key  and
    -- ceh_ca_name = ceh_rec.ceh_ca_name and
    -- ceh_wfl_name = ceh_rec.ceh_wfl_name  and  
    -- ceh_user_name = ceh_rec.ceh_user_name  and
    -- ceh_step_path  = ceh_rec.ceh_step_path and
    -- ceh_event = ceh_rec.ceh_event  and
    -- ceh_data  = ceh_rec.ceh_data
		); 
		
		-- restore row
		insert into ceh values (                                       
    ceh_rec.ceh_cmp_id,
    ceh_rec.ceh_id  ,
    ceh_rec.ceh_date  ,
    ceh_rec.ceh_entry_key  ,
    ceh_rec.ceh_ca_name ,
    ceh_rec.ceh_wfl_name  ,
    ceh_rec.ceh_user_name  ,
    ceh_rec.ceh_step_path  ,
    ceh_rec.ceh_event ,
    ceh_rec.ceh_data
		); 

		
	END LOOP;


	CLOSE my_cv;
	commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_ceh');
END;
/

CREATE OR REPLACE PROCEDURE RESTORE_wee(clnCoaId INTEGER)
IS
	CURSOR my_cv is select * from wee_clncoa where coa_id = clnCoaId;
	wee_rec wee_clncoa%ROWTYPE;
BEGIN
	DBMS_OUTPUT.PUT_LINE(' ');
	DBMS_OUTPUT.PUT_LINE('*** BEGIN RESTORE OF wee TABLE');

	-- show some primary keys
	OPEN my_cv;
	LOOP
		FETCH my_cv INTO wee_rec;
		EXIT WHEN my_cv%NOTFOUND;
		DBMS_OUTPUT.PUT_LINE('Next wee row = ' || wee_rec.wee_entry_id);
	
		-- delete existing row if any
		delete from wee where (                                        
    wee_event_id = wee_rec.wee_event_id  and
    wee_entry_id = wee_rec.wee_entry_id
    -- and wee_event_entry_status = wee_rec.wee_event_entry_status -- XXX failing string compare
		); 
		
		-- restore row
		insert into wee values (                                       
    wee_rec.wee_event_id,
    wee_rec.wee_entry_id,
    wee_rec.wee_event_entry_status
		); 
	END LOOP;
	CLOSE my_cv;
    commit;

EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_wee');
END;
/

CREATE OR REPLACE PROCEDURE RESTORE_all(clnCoaId INTEGER)
IS
	clnCAName varchar2(100);
	clnSrcCtgId number(9);
	clnCoaCtgId number(9);
BEGIN
  clnSrcCtgId := GET_SRC_CTG_ID_FROM_CA_ID(clnCoaId);
  clnCAName := GET_CA_NAME_FROM_ID(clnCoaId);
  clnCoaCtgId := GET_CA_CATALOG_ID_FROM_CA_ID(clnCoaId);

RESTORE_wee(clnCoaId);
RESTORE_ceh(clnCAName);
RESTORE_cel(clnCoaId);
RESTORE_cem(clnCoaId);
RESTORE_cae(clnCoaId);
RESTORE_itm(clnCoaCtgId);
RESTORE_ita(clnCoaCtgId);
RESTORE_icm(clnCoaCtgId); 
RESTORE_itmrsv(clnSrcCtgId); -- assumes source items only checked out to one collab area


EXCEPTION
	WHEN OTHERS THEN
		DBMS_OUTPUT.PUT_LINE('Error in proc RESTORE_all');
END;
/


CREATE OR REPLACE FUNCTION GET_CATALOG_ID_FROM_NAME(ctg_name VARCHAR2) RETURN NUMBER IS
    out_id NUMBER := 0;
    TYPE cur_typ IS REF CURSOR;
    c cur_typ;
    query_str VARCHAR2(200);
BEGIN
    query_str := 'select distinct ctg_id from ctg where ctg_name = :name';
    OPEN c FOR query_str USING ctg_name;
    LOOP
        FETCH c INTO out_id;
        EXIT WHEN c%NOTFOUND;
    END LOOP;
    CLOSE c;
   RETURN out_id;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END GET_CATALOG_ID_FROM_NAME;
/


CREATE OR REPLACE FUNCTION GET_COLLAB_ID_FROM_NAME(ca_name VARCHAR2) RETURN NUMBER IS
    out_id NUMBER := 0;
    TYPE cur_typ IS REF CURSOR;
    c cur_typ;
    query_str VARCHAR2(200);
BEGIN
    query_str := 'select distinct coa_id from coa where coa_name = :name';
    OPEN c FOR query_str USING ca_name;
    LOOP
        FETCH c INTO out_id;
        EXIT WHEN c%NOTFOUND;
    END LOOP;
    CLOSE c;
   RETURN out_id;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END GET_COLLAB_ID_FROM_NAME;
/



CREATE OR REPLACE PROCEDURE REPORT_VIEWS
IS
-- The Views are assumed to exist and contain data for the desired collaboration area.
BEGIN 
  DBMS_OUTPUT.enable(1000000);
COUNT_SHOW_SOME_PK_VIEW('SRC_CHECKED_OUT_PRIMARY_KEY', 'Primary keys of all source catalog items checked out to this collab area'); 
COUNT_SHOW_SOME_PK_VIEW('CA_ALL_PRIMARY_KEY', 'Primary keys of all items in the collaboration area catalog'); 
COUNT_SHOW_SOME_PK_VIEW('SRC_ALL_PRIMARY_KEY', 'All primary keys of source catalog items'); 
COUNT_SHOW_SOME_ITEM_ID_VIEW('ERROR_EVENT_ITEM_ID', 'Item ids of all collaboration area items in workflow events with ERROR status'); 
COUNT_SHOW_SOME_ITEM_ID_VIEW('NEW_EVENT_ITEM_ID', 'Item ids of all collaboration area items in workflow events with NEW status'); 
COUNT_SHOW_SOME_ITEM_ID_VIEW('DONE_EVENT_ITEM_ID', 'Item ids of all collaboration area items in workflow events with DONE status'); 
COUNT_SHOW_SOME_ITEM_ID_VIEW('PROCESSING_EVENT_ITEM_ID', 'Item ids of all collaboration area items in workflow events with PROCESSING status'); 
COUNT_SHOW_SOME_ITEM_ID_VIEW('GOOD_EVENT_ITEM_ID', 'Item ids of all collaboration area and source items that are only in NEW or DONE workflow events'); 
-- COUNT_SHOW_SOME_PK_VIEW('SRC_PRIMARY_KEY_IN_CEM', 'Primary keys of source items with corresponding CA items in CEM table'); 
COUNT_SHOW_SOME_PK_VIEW('SRC_PRIMARY_KEY_IN_CEM_CAE', 'Primary keys of source items with corresponding CA items that are in both the CEM and CAE tables in a state of integrity and have only good events');
COUNT_SHOW_SOME_PK_VIEW('SRC_PRIMARY_KEY_NOT_IN_CEM_CAE', 'All source catalog primary keys not in SRC_PRIMARY_KEY_IN_CEM_CAE'); 
COUNT_SHOW_SOME_PK_VIEW('SRC_PRIMARY_KEY_EVENT_ONLY', 'Primary keys of source items absent from CAE, CEM, or both, but in either a NEW or DONE event'); 
COUNT_SHOW_SOME_PK_VIEW('SRC_PRIMARY_KEY_OK_CEL', 'Primary keys of checked out source items which either have no rows, or only good rows, in CEL');
COUNT_SHOW_SOME_PK_VIEW('SRC_GOOD_PRIMARY_KEY', 'Primary keys of checked out source items which appear to be in a state of integrity');
COUNT_SHOW_SOME_PK_VIEW('SRC_BAD_PRIMARY_KEY', 'Primary keys of checked out source items already found NOT in a state of integrity'); 
COUNT_SHOW_SOME_PK_VIEW('ADD_PRIMARY_KEY', 'Primary keys of collab area items NOT the pk of any (good or bad) checked-out source item');
COUNT_SHOW_SOME_PK_VIEW('ADD_PRIMARY_KEY_IN_CEM_CAE', 'Primary keys of added CA items that are in both the CEM and CAE tables in a state of integrity and have only good events'); 
COUNT_SHOW_SOME_PK_VIEW('ADD_PRIMARY_KEY_NOT_IN_CEM_CAE', 'Primary keys of added CA items that are not in ADD_PRIMARY_KEY_IN_CEM_CAE'); 
COUNT_SHOW_SOME_PK_VIEW('ADD_PRIMARY_KEY_EVENT_ONLY', 'Primary keys of added CA items that are NOT in CEM, CAE, but which have only good eventsXXX'); 
COUNT_SHOW_SOME_PK_VIEW('ADD_PRIMARY_KEY_OK_CEL', 'Primary keys of added CA items which either have no rows, or only good rows, in CEL');
COUNT_SHOW_SOME_PK_VIEW('ADD_GOOD_PRIMARY_KEY', 'Primary keys of added CA items which appear to be in a state of integrity');
COUNT_SHOW_SOME_PK_VIEW('ADD_BAD_PRIMARY_KEY', 'Primary keys of added CA items already found NOT in a state of integrity'); 
COUNT_SHOW_SOME_PK_VIEW('IN_INITIAL_PRIMARY_KEY', 'Primary keys of CA items in the workflow Initial step'); 
COUNT_SHOW_SOME_PK_VIEW('CA_ITM_NO_ITA_PRIMARY_KEY', 'Primary keys of ITM rows of CA catalog items with no corresponding ITA rows');
COUNT_SHOW_SOME_ITEM_ID_VIEW('CAE_ORPHAN_ITEM_ID', 'Item ids found in CAE with no corresponding ITM row');
COUNT_SHOW_SOME_ITEM_ID_VIEW('CEM_SRC_ORPHAN_ITEM_ID', 'Source catalog item ids found in CEM with no corresponding ITM row');
COUNT_SHOW_SOME_ITEM_ID_VIEW('CEM_CA_ORPHAN_ITEM_ID', 'CA catalog item ids found in CEM with no corresponding ITM row');
COUNT_SHOW_SOME_ITEM_ID_VIEW('CEL_CA_ORPHAN_ITEM_ID', 'CA catalog item ids found in CEL with no corresponding ITM row');
COUNT_SHOW_SOME_ITEM_ID_VIEW('WEE_CA_ORPHAN_ITEM_ID', 'CA catalog item ids found in WEE with no corresponding ITM row');
COUNT_SHOW_SOME_ITEM_ID_VIEW('CA_ITA_NO_ITM', 'CA catalog ITA rows with no corresponding ITM row');
COUNT_SHOW_SOME_ITEM_ID_VIEW('CA_ITM_NO_ITA', 'CA catalog ITM rows with no corresponding ITA rows');
COUNT_SHOW_SOME_PK_VIEW('TOTAL_BAD_PRIMARY_KEY', 'Complete list of primary keys found NOT to be in a state of integrity');
COUNT_SHOW_SOME_ITEM_ID_VIEW('TOTAL_BAD_ITEM_EVTQ', 'CA Item ids having workflow events in ERROR or PROCESSING or with orphan WEE rows');
COUNT_SHOW_SOME_ITEM_ID_VIEW('ADDITIONAL_BAD_CA_ITEM_ID', 'CA Item ids that are orphans, have bad events, or have ITA/ITM corruption');
-- COUNT_SHOW_SOME_ITEM_ID_VIEW('ADDITIONAL_BAD_SRC_ITEM_ID', 'Source items that are CEM orphans');
COUNT_SHOW_SOME_ITEM_ID_VIEW('TOTAL_BAD_CA_ITEM_ID', 'Complete list of CA Item ids NOT in a state of integrity');
COUNT_SHOW_SOME_ITEM_ID_VIEW('TOTAL_BAD_SRC_ITEM_ID', 'Complete list of source Item ids NOT in a state of integrity');
COUNT_SHOW_SOME_PK_VIEW('TOTAL_GOOD_PRIMARY_KEY', 'Complete list of primary keys in a state of integrity');
COUNT_SHOW_SOME_ITEM_ID_VIEW('TOTAL_GOOD_CA_ITEM_ID', 'Complete list of CA Item ids of CA Items in a state of integrity');
COUNT_SHOW_SOME_ITEM_ID_VIEW('TOTAL_GOOD_SRC_ITEM_ID', 'Complete list of source Item ids in a state of integrity');


  DBMS_OUTPUT.Put_Line(' ');
  DBMS_OUTPUT.Put_Line('*** REPORT - FULL LIST OF PRIMARY KEYS OF GOOD ITEMS, WHOSE PRIMARY KEYS ARE KNOWN');
  SHOW_ALL_PK_VIEW( 'TOTAL_GOOD_PRIMARY_KEY', 'All bad Items, by primary key.' );
  DBMS_OUTPUT.Put_Line(' ');
  DBMS_OUTPUT.Put_Line('*** REPORT - FULL LIST OF PRIMARY KEYS OF BAD ITEMS, WHOSE PRIMARY KEYS ARE KNOWN');
  SHOW_ALL_PK_VIEW( 'TOTAL_BAD_PRIMARY_KEY', 'All bad Items, by primary key.' );
  DBMS_OUTPUT.Put_Line(' ');
  DBMS_OUTPUT.Put_Line('*** REPORT - FULL LIST OF ITEM IDS OF BAD COLLABORATION AREA ITEMS, SOME KNOWN ALSO BY PK, SOME KNOWN ONLY BY ITEM ID');
  SHOW_ALL_IID_VIEW( 'TOTAL_BAD_CA_ITEM_ID', 'All bad CA Items, by item id.' );
  DBMS_OUTPUT.Put_Line(' ');
  DBMS_OUTPUT.Put_Line('*** REPORT - FULL LIST OF ITEM IDS OF BAD SOURCE CATALOG ITEMS, SOME KNOWN ALSO BY PK, SOME KNOWN ONLY BY ITEM ID');
  SHOW_ALL_IID_VIEW( 'TOTAL_BAD_SRC_ITEM_ID', 'All bad SRC Items, by item id.' );
  DBMS_OUTPUT.Put_Line(' ');
  DBMS_OUTPUT.Put_Line('*** DONE WITH REPORTS');
  COMMIT; 
END; 
/

CREATE OR REPLACE FUNCTION GET_CA_CATALOG_ID_FROM_CA_ID(clnCoaId INTEGER) RETURN NUMBER IS
	TYPE MyCurTyp IS REF CURSOR;
	clnCoaCtgId number(9);
        c MyCurTyp;
BEGIN
  OPEN c FOR 'select COA_DST_CONTAINER_ID from coa where coa_id = ' || clnCoaId; FETCH c INTO clnCoaCtgId;
    CLOSE c;
   RETURN clnCoaCtgId;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END GET_CA_CATALOG_ID_FROM_CA_ID;
/

CREATE OR REPLACE FUNCTION GET_SRC_CTG_ID_FROM_CA_ID(clnCoaId INTEGER) RETURN NUMBER IS
	TYPE MyCurTyp IS REF CURSOR;
	clnSrcCtgId number(9);
        c MyCurTyp;
BEGIN
  OPEN c FOR 'select COA_SRC_CONTAINER_ID from coa where coa_id = ' || clnCoaId; FETCH c INTO clnSrcCtgId;
    CLOSE c;
   RETURN clnSrcCtgId;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END GET_SRC_CTG_ID_FROM_CA_ID;
/


CREATE OR REPLACE FUNCTION GET_CA_NAME_FROM_ID(clnCoaId INTEGER) RETURN varchar2 IS
	TYPE MyCurTyp IS REF CURSOR;
	clnCAName varchar2(100);
        c MyCurTyp;
BEGIN
    OPEN c FOR 'select COA_NAME from coa where coa_id = ' || clnCoaId; FETCH c INTO clnCAName;
    CLOSE c;
   RETURN clnCAName;
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
       -- Consider logging the error and then re-raise
       RAISE;
END GET_CA_NAME_FROM_ID;
/


CREATE OR REPLACE PROCEDURE DELETE_CA_FROM_BACKUP_TABLES(clnCoaId INTEGER)
IS
-- The Views are assumed to exist and contain data for the desired collaboration area.
	clnCAName varchar2(100);
	clnSrcCtgId number(9);
	clnCoaCtgId number(9);
BEGIN
  clnSrcCtgId := GET_SRC_CTG_ID_FROM_CA_ID(clnCoaId);
  clnCAName := GET_CA_NAME_FROM_ID(clnCoaId);
  clnCoaCtgId := GET_CA_CATALOG_ID_FROM_CA_ID(clnCoaId);

  DBMS_OUTPUT.enable(1000000);

delete ITMRSV_CLNCOA where itm_container_id = clnSrcCtgId;
delete ITM_CLNCOA where itm_container_id = clnCoaCtgId;
delete ITA_CLNCOA where ita_catalog_id = clnCoaCtgId;
delete ICM_CLNCOA where icm_catalog_id = clnCoaCtgId;
delete CEM_CLNCOA where cem_ca_id = clnCoaId;
delete CAE_CLNCOA where cae_ca_id = clnCoaId;
delete CEH_CLNCOA where ceh_ca_name = clnCAName;
delete CEL_CLNCOA where cel_container_id = clnCoaId;
delete WEE_CLNCOA where coa_id = clnCoaId;
  
COMMIT; 

END; 
/


CREATE OR REPLACE PROCEDURE CLEAN_AND_BACKUP_DATABASE_ROWS(clnCoaId INTEGER)
IS
-- The Views are assumed to exist and contain data for the desired collaboration area.
	clnSrcCtgId number(9);
	clnCoaCtgId number(9);
BEGIN
  clnSrcCtgId := GET_SRC_CTG_ID_FROM_CA_ID(clnCoaId);
  clnCoaCtgId := GET_CA_CATALOG_ID_FROM_CA_ID(clnCoaId);

  DBMS_OUTPUT.enable(1000000);

-- Remove pre-existing backup rows for this collab area, if any
DELETE_CA_FROM_BACKUP_TABLES(clnCoaId);


-- Now clean and backup

  CLEAN_ITEMS_OF_PK_VIEW('TOTAL_BAD_PRIMARY_KEY', clnCoaId, clnSrcCtgId, clnCoaCtgId);

  CLEAN_CA_ITEMS_OF_IID_VIEW('ADDITIONAL_BAD_CA_ITEM_ID', clnCoaId, clnCoaCtgId);    -- note:  COLLAB AREA CATALOG ID ONLY

  -- temporarily use just CEM_SRC_ORPHAN_ITEM_ID in place of ADDITIONAL_BAD_SRC_ITEM_ID (see create_views.sql)
  -- CLEAN_SRC_ITEMS_OF_IID_VIEW('ADDITIONAL_BAD_SRC_ITEM_ID', clnCoaId, clnSrcCtgId); -- note:  SOURCE CATALOG ID ONLY
  CLEAN_SRC_ITEMS_OF_IID_VIEW('CEM_SRC_ORPHAN_ITEM_ID', clnCoaId, clnSrcCtgId); -- note:  SOURCE CATALOG ID ONLY


  DBMS_OUTPUT.Put_Line(' ');
  DBMS_OUTPUT.Put_Line('*********** DONE WITH CLEAN ************************');

  COMMIT; 
END; 
/


